﻿using System;
using System.Configuration;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ServiceModel;
using MySite.SalesforceService;

namespace MySite.Models
{
    class SalesforceUtil : IDisposable
    {
        private SoapClient client;
        private SessionHeader header;

        /// <summary>
        /// Create a utility and log in
        /// </summary>
        public SalesforceUtil()
        {
            string username = ConfigurationManager.AppSettings["SalesforceUsername"];
            string password = ConfigurationManager.AppSettings["SalesforcePassword"];

            // Create a SoapClient specifically for logging in
            var lr = new SoapClient().login(null, username, password);
            if (lr.passwordExpired)
            {
                throw new Exception("An error has occurred. Your password has expired.");
            }

            // On successful login, cache session info and API endpoint info
            var endpoint = new EndpointAddress(lr.serverUrl);
            header = new SessionHeader();
            header.sessionId = lr.sessionId;
            client = new SoapClient("Soap", endpoint);
        }

        /// <summary>
        /// Log out
        /// </summary>
        public void Dispose()
        {
            client.logout(header);
        }


        /// <summary>
        /// To determine the objects that are available to the logged-in user
        /// </summary>
        /// <returns></returns>
        public DescribeGlobalResult DescribeGlobal()
        {
            // describeGlobal() returns an array of object results that  
            // includes the object names that are available to the logged-in user. 
            DescribeGlobalResult dgr;
            client.describeGlobal(header, null, out dgr);
            return dgr;
        }

        /// <summary>
        /// Get information about an object type
        /// </summary>
        /// <param name="objectType"></param>
        /// <returns></returns>
        public DescribeSObjectResult DescribeObject(string objectType)
        {
            // Call describeSObjects() passing in an array with one object type name 
            DescribeSObjectResult[] dsrArray;
            var liArray = client.describeSObjects(header, null, null, new string[] { objectType }, out dsrArray);
            // Since we described only one sObject, we should have only
            // one element in the DescribeSObjectResult array.
            return dsrArray[0];
        }

        private QueryResult GetQuery(String soqlQuery)
        {
            QueryResult qr;
            client.query(header, null, null, null, soqlQuery, out qr);
            return qr;
        }

        private T Retrieve<T>(string objectType, string fieldList, string id)
        {
            var results = Retrieve<T>(objectType, fieldList, new string[] { id });
            return results.Count > 0 ? results[0] : default(T);
        }

        private List<T> Retrieve<T>(string objectType, string fieldList, string[] ids)
        {
            sObject[] result;
            client.retrieve(header, null, null, null, fieldList, objectType, ids, out result);
            return result.Cast<T>().ToList();
        }

        private SaveResult[] Create(sObject[] sObjects)
        {
            LimitInfo[] limitInfoHeader;
            SaveResult[] result;
            client.create(header, null, null, null, null, null, null, null, null, null, null, null,
                sObjects, out limitInfoHeader, out result);
            return result;
        }

        private SaveResult[] Update(sObject[] sObjects)
        {
            LimitInfo[] limitInfoHeader;
            SaveResult[] result;
            client.update(header, null, null, null, null, null, null, null, null, null, null, null, null,
                sObjects, out limitInfoHeader, out result);
            return result;
        }

        public List<Contact> GetAllContacts()
        {
            var qr = GetQuery("SELECT Id, FirstName, LastName FROM Contact LIMIT 100");
            return qr.records.Cast<Contact>().ToList();
        }

        public string GetContactIdByEmail(string email)
        {
            var qr = GetQuery(string.Format("SELECT Id FROM Contact WHERE Email = '{0}'", email));
            return qr.records != null && qr.records.Count() > 0 ? ((Contact)qr.records[0]).Id : null;
        }

        public SaveResult[] SaveContact(Contact contact)
        {
            if (string.IsNullOrEmpty(contact.Id))
            {
                return Create(new sObject[] { contact });
            }
            else
            {
                return Update(new sObject[] { contact });
            }
        }

        public SaveResult[] SaveLead(Lead lead)
        {
            if (string.IsNullOrEmpty(lead.Id))
            {
                return Create(new sObject[] { lead });
            }
            else
            {
                return Update(new sObject[] { lead });
            }
        }

        private List<Contact> GetContacts(string soqlQuery)
        {
            var qr = GetQuery(soqlQuery);
            return qr.records.Cast<Contact>().ToList();
        }
    }
}